<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://mycatchers.com
 * @since      1.0.0
 *
 * @package    Catchers_Helpdesk_Addon1
 * @subpackage Catchers_Helpdesk_Addon1/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Catchers_Helpdesk_Addon1
 * @subpackage Catchers_Helpdesk_Addon1/includes
 * @author     mycatchers <support@mycatchers.com>
 */
class Catchers_Helpdesk_Addon1 {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Catchers_Helpdesk_Addon1_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct($pluginName) {

		$this->plugin_name = $pluginName;
		$this->version = STG_CATCHERS_HELPDESK_ADDON1_VERSION;

		$this->load_dependencies();
		$this->set_locale();

		$depStatus = self::checkPluginDeps();

		if(!$depStatus)
		{
            define('STG_CATCHERS_HELPDESK_ADDON1_TEXT_DOMAIN_NAME', 'catchers-helpdesk');
			add_filter( 'plugin_row_meta', array($this, 'pluginRowMeta' ), 10, 2 );
			add_action('admin_init', array($this, 'redirectToWelcomePage'), 12, 0);
			add_action('admin_menu', array($this, 'registerMenu'));
		}
        else{
            if(defined('STG_HELPDESK_POST_TYPE') && $depStatus){
                define('STG_CATCHERS_HELPDESK_ADDON1_TEXT_DOMAIN_NAME', STG_HELPDESK_TEXT_DOMAIN_NAME);
                $this->define_admin_hooks();
                $this->define_public_hooks();
            }
        }


	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Catchers_Helpdesk_Addon1_Loader. Orchestrates the hooks of the plugin.
	 * - Catchers_Helpdesk_Addon1_i18n. Defines internationalization functionality.
	 * - Catchers_Helpdesk_Addon1_Admin. Defines all hooks for the admin area.
	 * - Catchers_Helpdesk_Addon1_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-catchers-helpdesk-addon1-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-catchers-helpdesk-addon1-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-catchers-helpdesk-addon1-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-catchers-helpdesk-addon1-public.php';

		$this->loader = new Catchers_Helpdesk_Addon1_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Catchers_Helpdesk_Addon1_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Catchers_Helpdesk_Addon1_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Catchers_Helpdesk_Addon1_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		$helpdeskInit = \StgHelpdesk\Core\Stg_Helpdesk_Init::getInstance();

		add_action('restrict_manage_posts', function () {
			\StgHelpdesk\Ticket\Stg_Helpdesk_TicketList::categoriesFilter();
		}, 11, 0);

		add_action('ticket_category_edit_form_fields', function ($tag) {
			\StgHelpdesk\Ticket\Stg_Helpdesk_TicketCategory::addCustomFields($tag);
		}, 10, 2);


		add_action('edited_ticket_category', function ($term_id) {
			\StgHelpdesk\Ticket\Stg_Helpdesk_TicketCategory::saveCustomFieldsValue($term_id);
		}, 10, 2);

		add_action('init', array($helpdeskInit, 'create_taxonomy_custom_fields'));
		add_action('init', 'stgh_hide_sb_yoast');
		add_action('customfields_add_form_fields', array($helpdeskInit, 'customfields_taxonomy_add_new_meta_field'), 10, 2 );
		add_action('customfields_edit_form_fields', array($helpdeskInit, 'customfields_taxonomy_edit_new_meta_field'), 10, 2 );
		add_action('edited_customfields', array($helpdeskInit, 'save_taxonomy_custom_meta'), 10, 2 );
		add_action('create_customfields', array($helpdeskInit, 'save_taxonomy_custom_meta'), 10, 2 );
		add_filter('manage_customfields_custom_column', array($helpdeskInit,'add_customfields_column_content'),10,3);
		add_filter('manage_edit-customfields_columns', array($helpdeskInit,'add_customfields_column_header'), 10);
		add_filter('customfields_row_actions', array($helpdeskInit, 'remove_row_actions'), 10, 2 );
		add_filter('pre_delete_term', array($helpdeskInit, 'delete_customfields'), 10, 3 );

		//add custom forms and metas
		add_action('init', array($helpdeskInit, 'create_taxonomy_custom_forms'));
		add_action('customforms_add_form_fields', array($helpdeskInit, 'customforms_taxonomy_add_new_meta_field'), 10, 2 );
		add_filter('manage_customforms_custom_column', array($helpdeskInit,'add_customforms_column_content'),10,3);
		add_filter('manage_edit-customforms_columns', array($helpdeskInit,'add_customforms_column_header'), 10);
		add_filter('customforms_row_actions', array($helpdeskInit, 'remove_row_actions'), 10, 2 );
		add_action('edited_customforms', array($helpdeskInit, 'save_taxonomy_custom_meta'), 10, 2 );
		add_action('create_customforms', array($helpdeskInit, 'save_taxonomy_custom_meta'), 10, 2 );


		//add tinymce in custom forms
		add_filter('customforms_edit_form_fields', array($helpdeskInit, 'customforms_edit_cat_description'));
		add_action('admin_head', array($helpdeskInit, 'taxonomy_tinycme_hide_description'));
		remove_filter('pre_term_description', 'wp_filter_kses');
		remove_filter('term_description', 'wp_kses_data');


		add_filter('stgh_plugin_settings', array($this,'stgh_updates_integrations'), 110, 1);

		//run updater
		add_action( 'init', array( &$this, 'run_updater' ) );
		add_filter('stgh_table_column', array($this,'stgh_table_column'));
	}


	function stgh_table_column($columns){
		$fields = get_terms( array(
			'taxonomy' => 'customfields',
			'hide_empty' => false,
		));

		foreach($fields as $field)
		{
			$columns[STG_HELPDESK_POST_TYPE . '-customfields'.$field->term_id] = $field->name;
		}


		return $columns;
	}

	function stgh_updates_integrations($def)
	{

		$settings = array(
			'integrations' => array(
				'name' => __('Integrations', STG_CATCHERS_HELPDESK_ADDON1_TEXT_DOMAIN_NAME),
				'options' => array(
					array(
						'type' => 'heading',
						'name' => __('Addon\'s update', STG_CATCHERS_HELPDESK_ADDON1_TEXT_DOMAIN_NAME)
					),
					array(
						'name' => __('Envato Personal Token', STG_CATCHERS_HELPDESK_ADDON1_TEXT_DOMAIN_NAME),
						'id' => 'envato_token',
						'type' => 'text',
					),
					array(
						'name' => __('Plugin license', STG_CATCHERS_HELPDESK_ADDON1_TEXT_DOMAIN_NAME),
						'id' => 'plugin_license',
						'type' => 'text',
					),
					array(
						'type' => 'heading',
						'name' => __('Custom forms settings', STG_CATCHERS_HELPDESK_ADDON1_TEXT_DOMAIN_NAME)
					),
					array(
						'name' => __('Allow Helpdesk Managers manage custom fields', STG_CATCHERS_HELPDESK_ADDON1_TEXT_DOMAIN_NAME),
						'id' => 'man_rules',
						'type' => 'checkbox',
					),
				)
			),
		);


		$res = array_merge($def, $settings);

		if(isset($def['integrations']['options']))
		{
			$res['integrations']['options'] = array_unique(array_merge($def['integrations']['options'], $settings['integrations']['options']), SORT_REGULAR);
		}


		return $res;

	}

	function registerMenu(){
			add_submenu_page('index.php',
				__('Catchers Helpdesk  Add-on starting page.', STG_CATCHERS_HELPDESK_ADDON1_TEXT_DOMAIN_NAME), __('Welcome', STG_CATCHERS_HELPDESK_ADDON1_TEXT_DOMAIN_NAME), 'activate_plugins',
				'stgh-addon1-start', array($this, 'showWelcomePage'));

			remove_submenu_page('index.php', 'stgh-addon1-start');
	}

	function redirectToWelcomePage(){
		if(get_option('stgh_redirect_to_welcome_page_addon'))
		{
			wp_redirect( add_query_arg(array('page' => 'stgh-addon1-start'),admin_url('index.php')));
			delete_option('stgh_redirect_to_welcome_page_addon');
		}
	}

	public function showWelcomePage()
	{
		/** @noinspection PhpIncludeInspection */
		include_once(STG_CATCHERS_HELPDESK_ADDON1_ROOT . 'admin/partials/welcome.php');
	}


	public function pluginRowMeta( $links, $file ) {
		list($part1,$part2) = explode('/',$file);
		if ( STG_CATCHERS_HELPDESK_ADDON1_NAME == $part1 &&  STG_CATCHERS_HELPDESK_ADDON1_NAME.".php" == $part2) {

			$freeHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME.'/'.STG_CATCHERS_HELPDESK_NAME.'.php');
			$proHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME_PRO . '/' . STG_CATCHERS_HELPDESK_NAME_PRO . '.php');
			$addonDeps = Catchers_Helpdesk_Addon1::checkPluginDeps();

			$welcomePage = add_query_arg(array('page' => 'stgh-addon1-start'),admin_url('index.php'));
			$row_meta = array();

			if(!$freeHDActivated && !$proHDActivated){
				$row_meta = array(
					'welcome'    => '<a class="error-message" href="' . esc_url($welcomePage) . '">' . esc_html__( 'Attention, please! Catchers Helpdesk plugin is not installed or not activated!', STG_CATCHERS_HELPDESK_ADDON1_TEXT_DOMAIN_NAME ) . '</a>'
				);

			}else{
				if (!$addonDeps) {
					$row_meta = array(
						'welcome'    => '<a class="error-message" href="' . esc_url($welcomePage) . '">' . esc_html__( 'Attention, please! You have the old version of Catchers Helpdesk!', STG_CATCHERS_HELPDESK_ADDON1_TEXT_DOMAIN_NAME ) . '</a>'
					);

				}
			}


			return array_merge( $links, $row_meta );
		}

		return (array) $links;
	}




	function run_updater(){
		$pluginPath = STG_CATCHERS_HELPDESK_ADDON1_ROOT."".STG_CATCHERS_HELPDESK_ADDON1_NAME.".php";
		$pluginData = stgh_wc_updater_get_plugin_data($pluginPath);
		$pluginCurrentVersion = $pluginData['Version'];

		$adapterName = 'Mycatchers';
		$data = array(
			'Mycatchers' => array(
				'license' => 'none',
			)
		);


		$pluginLicense = stgh_get_option('plugin_license');
		$envatoToken = stgh_get_option('envato_token');


		if(strlen($pluginLicense) == 34){
			$adapterName = 'Mycatchers';
			$data = array(
				'Mycatchers' => array(
					'license' => $pluginLicense
				)
			);
		}else{
			if(!empty($envatoToken))
			{
				$adapterName = 'Envato';
				$data = array(
					'Envato' => array(
						'license' => 'envato',
						'token' => $envatoToken,
						// @todo
						'itemId' => ''
					)
				);
			}
		}
		if($adapterName != false){
			if(!class_exists('\StgHelpdeskAddon1\Updater\AutoUpdater'))
			{
				/** @noinspection PhpIncludeInspection */
				require_once plugin_dir_path( dirname( __FILE__ ) ) . 'updater/auto_updater.php';
			}
			new \StgHelpdeskAddon1\Updater\AutoUpdater($adapterName, $pluginCurrentVersion, $data, $this->get_plugin_name());
		}

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Catchers_Helpdesk_Addon1_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Catchers_Helpdesk_Addon1_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

	public static function checkPluginDeps()
	{
		include_once(ABSPATH.'wp-admin/includes/plugin.php');

		$freeHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME . '/' . STG_CATCHERS_HELPDESK_NAME . '.php');
		$proHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME_PRO . '/' . STG_CATCHERS_HELPDESK_NAME_PRO . '.php');

		if (!$freeHDActivated && !$proHDActivated) {
			return false;
		} else {
			$pluginFile = STG_HELPDESK_ROOT.STG_HELPDESK_NAME.".php";
			$readmeFile = STG_HELPDESK_ROOT."README.txt";
			$pluginData = stgh_wc_updater_get_plugin_data($pluginFile, $readmeFile);

			if (version_compare($pluginData["Version"], STG_CATCHERS_HELPDESK_ADDON1_VERSION, '<')) {
				return false;
			}

		}

		return true;
	}

}
