<?php

use StgHelpdesk\Core\Stg_Helpdesk_Init;

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://mycatchers.com
 * @since      1.0.0
 *
 * @package    Catchers_Helpdesk_Addon6
 * @subpackage Catchers_Helpdesk_Addon6/includes
 */


/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Catchers_Helpdesk_Addon6
 * @subpackage Catchers_Helpdesk_Addon6/includes
 * @author     mycatchers <support@mycatchers.com>
 */
class Catchers_Helpdesk_Addon6 {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Catchers_Helpdesk_Addon6_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct($pluginName) {

		$this->plugin_name = $pluginName;
		$this->version = STG_CATCHERS_HELPDESK_ADDON6_VERSION;


		$this->load_dependencies();
		$this->set_locale();

		$depStatus = self::checkPluginDeps();

		if(!$depStatus) {
            define('STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME', 'catchers-helpdesk');
			add_filter('plugin_row_meta', array($this, 'pluginRowMeta'), 10, 2);
			add_action('admin_init', array($this, 'redirectToWelcomePage'), 12, 0);
			add_action('admin_menu', array($this, 'registerMenu'));
		}
        else{
            if(defined('STG_HELPDESK_POST_TYPE') && $depStatus){
                if(defined('STG_HELPDESK_TEXT_DOMAIN_NAME'))
                {
                    define('STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME', STG_HELPDESK_TEXT_DOMAIN_NAME);
                }

                $this->define_admin_hooks();
                $this->define_public_hooks();
            }
        }



	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Catchers_Helpdesk_Addon6_Loader. Orchestrates the hooks of the plugin.
	 * - Catchers_Helpdesk_Addon6_i18n. Defines internationalization functionality.
	 * - Catchers_Helpdesk_Addon6_Admin. Defines all hooks for the admin area.
	 * - Catchers_Helpdesk_Addon6_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-catchers-helpdesk-addon6-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-catchers-helpdesk-addon6-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-catchers-helpdesk-addon6-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-catchers-helpdesk-addon6-public.php';

        $this->loader = new Catchers_Helpdesk_Addon6_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Catchers_Helpdesk_Addon6_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Catchers_Helpdesk_Addon6_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Catchers_Helpdesk_Addon6_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

        add_action('admin_menu', array($this, 'add_kb_menu'),10);

        $helpdeskInit = Stg_Helpdesk_Init::getInstance();

        add_filter('savedreply_add_form_fields', array($helpdeskInit, 'savedreply_add_form_fields'));
        add_action('edited_savedreply', array($helpdeskInit, 'save_taxonomy_custom_meta'), 10, 2 );
        add_action('create_savedreply', array($helpdeskInit, 'save_taxonomy_custom_meta'), 10, 2 );

        add_filter('manage_savedreply_custom_column', array($helpdeskInit,'add_savedreply_column_content'),10,3);
        add_filter('manage_edit-savedreply_columns', array($helpdeskInit,'add_savedreply_column_header'), 10);


        add_action('stgh_qa_edit_savedreply', array($this, 'stgh_qa_edit_savedreply'), 10, 1 );

        //run updater
		add_action( 'init', array( &$this, 'run_updater' ) );

        /** @noinspection PhpIncludeInspection */
        require_once(STG_HELPDESK_PUBLIC . '/shortcodes/stg-shortcode-kb.php');
	}

    function stgh_qa_edit_savedreply($values){
        $value = (isset($values['custom_field_toqa_meta']))? $values['custom_field_toqa_meta']:'';
        ?>
        <tr class="form-field">
            <th scope="row" valign="top"><label for="stgh_term_meta[custom_field_toqa_meta]"><?php _e( 'Convert into public Q&A note', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME ); ?></label></th>
            <td>
                <input <?=($value == 'on')? "checked=checked":""?> type="checkbox" name="stgh_term_meta[custom_field_toqa_meta]" id="stgh_term_meta[custom_field_toqa_meta]"/>
            </td>
        </tr>
        <?php $value = (isset($values['custom_field_topic_meta']))? $values['custom_field_topic_meta']:'';?>
        <tr class="form-field">
            <th scope="row" valign="top"><label for="stgh_term_meta[custom_field_topic_meta]"><?php _e( 'Topic', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME ); ?></label></th>
            <td>
                <input value="<?=$value?>" type="text" name="stgh_term_meta[custom_field_topic_meta]" id="stgh_term_meta[custom_field_topic_meta]"/>
            </td>
        </tr>

        <?php $topics = stgh_qa_topics_list('stgh_qa_topics_list'); ?>
        <?php if(!empty($topics)){
            $class = '';
        }else{
            $class = 'stgh_hide';
        }
        ?>

        <?php $topics = stgh_qa_topics_list('stgh_qa_topics_list'); ?>
            <tr class="form-field <?=$class?>">
                <th scope="row" valign="top">
                </th>
                <td>
                    <label for="stgh_qa_topics_list">
                        <?php _e( 'or choose from the most used:', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME ); ?>
                        <?=$topics?>
                    </label>
                </td>
            </tr>
        <?php
	}

    public function add_kb_menu()
    {
        global $submenu;

	    if(!isset($submenu['edit.php?post_type=stgh_ticket']))
		    return;

        foreach($submenu['edit.php?post_type=stgh_ticket'] as $key => $menu)
        {
            if($menu[2] == 'kb')
            {
	            unset($submenu['edit.php?post_type=stgh_ticket'][$key]);
            }
        }


        add_submenu_page( 'edit.php?post_type=' . STG_HELPDESK_POST_TYPE,
            __( 'Knowledge Base', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME ),
            __( 'Knowledge Base', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME ),
            'settings_tickets',
            'kb',
            array( $this, 'show_kb_page' )
        );

    }

    function show_kb_page(){
        ob_start();

        /** @noinspection PhpIncludeInspection */
        include_once(STG_CATCHERS_HELPDESK_ADDON6_ROOT . 'admin/partials/kb.php');
        $page =  ob_get_clean();
        echo $page;
    }


	function registerMenu(){
			add_submenu_page('index.php',
				__('Catchers Helpdesk  Add-on starting page.', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME), __('Welcome', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME), 'activate_plugins',
				'stgh-addon6-start', array($this, 'showWelcomePage'));

			remove_submenu_page('index.php', 'stgh-addon6-start');
	}

	function redirectToWelcomePage(){
		if(get_option('stgh_redirect_to_welcome_page_addon'))
		{
			wp_redirect( add_query_arg(array('page' => 'stgh-addon6-start'),admin_url('index.php')));
			delete_option('stgh_redirect_to_welcome_page_addon');
		}
	}

	public function showWelcomePage()
	{
		/** @noinspection PhpIncludeInspection */
		include_once(STG_CATCHERS_HELPDESK_ADDON6_ROOT . 'admin/partials/welcome.php');
	}

	public function pluginRowMeta( $links, $file ) {
		list($part1,$part2) = explode('/',$file);

		if ( STG_CATCHERS_HELPDESK_ADDON6_NAME == $part1 &&  STG_CATCHERS_HELPDESK_ADDON6_NAME.".php" == $part2) {

            include_once(ABSPATH.'wp-admin/includes/plugin.php');

            $freeHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME . '/' . STG_CATCHERS_HELPDESK_NAME . '.php');

            $addon2Activated = false;
            $hdOldVersion = $addon2OldVersion = false;

            if(defined('STG_CATCHERS_HELPDESK_ADDON2_NAME') && defined('STG_CATCHERS_HELPDESK_ADDON2_ROOT')) {
                $addon2Activated = is_plugin_active(STG_CATCHERS_HELPDESK_ADDON2_NAME . '/' . STG_CATCHERS_HELPDESK_ADDON2_NAME . '.php');

                $pluginFile = STG_CATCHERS_HELPDESK_ADDON2_ROOT . STG_CATCHERS_HELPDESK_ADDON2_NAME . ".php";
                $readmeFile = STG_CATCHERS_HELPDESK_ADDON2_ROOT . "README.txt";
                $pluginData = stgh_wc_updater_get_plugin_data($pluginFile, $readmeFile);

                if (version_compare($pluginData["Version"], STG_CATCHERS_HELPDESK_ADDON6_VERSION, '<')) {
                    $addon2OldVersion = true;
                }
            }

            $proHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME_PRO . '/' . STG_CATCHERS_HELPDESK_NAME_PRO . '.php');

            if(defined('STG_HELPDESK_ROOT') && defined('STG_HELPDESK_NAME')) {
                $pluginFile = STG_HELPDESK_ROOT . STG_HELPDESK_NAME . ".php";
                $readmeFile = STG_HELPDESK_ROOT . "README.txt";
                $pluginData = stgh_wc_updater_get_plugin_data($pluginFile, $readmeFile);

                if (version_compare($pluginData["Version"], STG_CATCHERS_HELPDESK_ADDON6_VERSION, '<')) {
                    $hdOldVersion = true;
                }
            }

			$welcomePage = add_query_arg(array('page' => 'stgh-addon6-start'),admin_url('index.php'));
			$row_meta = array();

			if(!$freeHDActivated && !$proHDActivated){
				$row_meta = array(
					'welcome'    => '<a class="error-message" href="' . esc_url($welcomePage) . '" >' . esc_html__( 'Attention, please! Catchers Helpdesk plugin is not installed or not activated!', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME ) . '</a>'
				);

			}

			$row_meta2 = array();

            if(!$addon2Activated){
                $row_meta2 = array(
                    'welcome2'    => '<a class="error-message" href="' . esc_url($welcomePage) . '" >' . esc_html__( 'Attention, please! Catchers Helpdesk Team Collaboration Add-on plugin is not installed or not activated!', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME ) . '</a>'
                );

            }

            $row_meta3 = array();

            if ($hdOldVersion) {
                $row_meta3 = array(
                    'welcome3'    => '<a class="error-message" href="' . esc_url($welcomePage) . '">' . esc_html__( 'Attention, please! You have the old version of Catchers Helpdesk!', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME ) . '</a>'
                );
            }


            $row_meta4 = array();

            if ($addon2OldVersion) {
                $row_meta = array(
                    'welcome4'    => '<a class="error-message" href="' . esc_url($welcomePage) . '">' . esc_html__( 'Attention, please! You have the old version of Catchers Helpdesk Team Collaboration Add-on!', STG_CATCHERS_HELPDESK_ADDON6_TEXT_DOMAIN_NAME ) . '</a>'
                );
            }


            return array_merge( $links, $row_meta,$row_meta2,$row_meta3,$row_meta4 );
		}

		return (array) $links;
	}


	function run_updater(){
		$pluginPath = STG_CATCHERS_HELPDESK_ADDON6_ROOT."".STG_CATCHERS_HELPDESK_ADDON6_NAME.".php";
		$pluginData = stgh_wc_updater_get_plugin_data($pluginPath);
		$pluginCurrentVersion = $pluginData['Version'];

		$adapterName = 'Mycatchers';
		$data = array(
			'Mycatchers' => array(
				'license' => 'none',
			)
		);

        $pluginLicense = false;
		if(function_exists('stgh_get_option')) {
            $pluginLicense = stgh_get_option('plugin_license');
            $envatoToken = stgh_get_option('envato_token');
        }

		if(strlen($pluginLicense) == 34){
			$adapterName = 'Mycatchers';
			$data = array(
				'Mycatchers' => array(
					'license' => $pluginLicense
				)
			);
		}else{
			if(!empty($envatoToken))
			{
				$adapterName = 'Envato';
				$data = array(
					'Envato' => array(
						'license' => 'envato',
						'token' => $envatoToken,
						//@todo
						'itemId' => ''
					)
				);
			}
		}
		if($adapterName != false){
			if(!class_exists('\StgHelpdeskAddon6\Updater\AutoUpdater'))
			{
				/** @noinspection PhpIncludeInspection */
				require_once plugin_dir_path( dirname( __FILE__ ) ) . 'updater/auto_updater.php';
			}
			new \StgHelpdeskAddon6\Updater\AutoUpdater($adapterName, $pluginCurrentVersion, $data, $this->get_plugin_name());
		}

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Catchers_Helpdesk_Addon6_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
	}


	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Catchers_Helpdesk_Addon6_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

	public static function checkPluginDeps()
	{
		include_once(ABSPATH.'wp-admin/includes/plugin.php');

		$freeHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME . '/' . STG_CATCHERS_HELPDESK_NAME . '.php');

        $addon2Activated = false;

        if(defined('STG_CATCHERS_HELPDESK_ADDON2_NAME'))
            $addon2Activated = is_plugin_active(STG_CATCHERS_HELPDESK_ADDON2_NAME . '/' . STG_CATCHERS_HELPDESK_ADDON2_NAME . '.php');

        $proHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME_PRO . '/' . STG_CATCHERS_HELPDESK_NAME_PRO . '.php');

		if (!($freeHDActivated && $addon2Activated) && !$proHDActivated) {
			return false;
		} else {
		    if(defined('STG_HELPDESK_ROOT') && defined('STG_HELPDESK_NAME')) {
                $pluginFile = STG_HELPDESK_ROOT . STG_HELPDESK_NAME . ".php";
                $readmeFile = STG_HELPDESK_ROOT . "README.txt";
                $pluginData = stgh_wc_updater_get_plugin_data($pluginFile, $readmeFile);

                if (version_compare($pluginData["Version"], STG_CATCHERS_HELPDESK_ADDON6_VERSION, '<')) {
                    return false;
                }
            }

		}

		return true;
	}

}
