<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://mycatchers.com
 * @since      1.0.0
 *
 * @package    Catchers_Helpdesk_Addon5
 * @subpackage Catchers_Helpdesk_Addon5/includes
 */

use StgHelpdesk\Core\Stg_Helpdesk_Init;
use StgHelpdesk\Ticket\Stg_Helpdesk_Ticket;

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Catchers_Helpdesk_Addon5
 * @subpackage Catchers_Helpdesk_Addon5/includes
 * @author     mycatchers <support@mycatchers.com>
 */
class Catchers_Helpdesk_Addon5 {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Catchers_Helpdesk_Addon5_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct($pluginName) {

		$this->plugin_name = $pluginName;
		$this->version = STG_CATCHERS_HELPDESK_ADDON5_VERSION;


		$this->load_dependencies();
		$this->set_locale();

		$depStatus = self::checkPluginDeps();

		if(!$depStatus) {
            define('STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME', 'catchers-helpdesk');
			add_filter('plugin_row_meta', array($this, 'pluginRowMeta'), 10, 2);
			add_action('admin_init', array($this, 'redirectToWelcomePage'), 12, 0);
			add_action('admin_menu', array($this, 'registerMenu'));
		}
        else{
            if(defined('STG_HELPDESK_POST_TYPE') && $depStatus){
                if(defined('STG_HELPDESK_TEXT_DOMAIN_NAME'))
                {
                    define('STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME', STG_HELPDESK_TEXT_DOMAIN_NAME);
                }

                if(get_option('stgh_need_delta')){
                    Catchers_Helpdesk_Addon5_Report::deltaTables();
                    delete_option('stgh_need_delta');
                }

                $this->define_admin_hooks();
                $this->define_public_hooks();
            }
        }



	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Catchers_Helpdesk_Addon5_Loader. Orchestrates the hooks of the plugin.
	 * - Catchers_Helpdesk_Addon5_i18n. Defines internationalization functionality.
	 * - Catchers_Helpdesk_Addon5_Admin. Defines all hooks for the admin area.
	 * - Catchers_Helpdesk_Addon5_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-catchers-helpdesk-addon5-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-catchers-helpdesk-addon5-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-catchers-helpdesk-addon5-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-catchers-helpdesk-addon5-public.php';

		/** @noinspection PhpIncludeInspection */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-catchers-helpdesk-addon5-report.php';

        /** @noinspection PhpIncludeInspection */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-catchers-helpdesk-addon5-report-cron.php';

        $this->loader = new Catchers_Helpdesk_Addon5_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Catchers_Helpdesk_Addon5_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Catchers_Helpdesk_Addon5_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Catchers_Helpdesk_Addon5_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

        //add_filter('stgh_plugin_settings', array($this,'stgh_core_helpcatcher'), 100, 1);

        add_action('wp_ajax_stgh_get_categories', array($this, 'get_categories'));
        add_action('wp_ajax_stgh_get_reports_data', array($this, 'stgh_get_reports_data'));
        add_action('wp_ajax_stgh_get_reports_data', array($this, 'stgh_get_reports_data'));
        add_action('wp_ajax_stgh_get_tasks', array($this, 'stgh_get_tasks'));

        add_action('stgh_create_default_report_hook', array($this,'stgh_create_default_report'));

        add_action('stgh_ticket_saved', function($params){

            $ticket = Stg_Helpdesk_Ticket::getInstance($params['id']);

            $ticketRow = Catchers_Helpdesk_Addon5_Report::getTicketPublicRow($ticket->getPost(), $params['source']);

            $ticketRow['event_type'] = 'new_ticket';

            Catchers_Helpdesk_Addon5_Report::addEventRowToTable($ticketRow);
        });

        add_action('stgh_ticket_reply_saved', function($params){

            $ticket = Stg_Helpdesk_Ticket::getInstance($params['id']);

            $ticketRow = Catchers_Helpdesk_Addon5_Report::getTicketPublicRow($ticket->getPost(), null);

            $ticketRow['event_type'] = $params['event_type'];
            $ticketRow['ticket_category'] = $params['category'];
            $ticketRow['ticket_assignee'] = $params['assignee'];

            Catchers_Helpdesk_Addon5_Report::addEventRowToTable($ticketRow);
        });


        add_action('stgh_ticket_category_saved', function($params){
            $ticket = Stg_Helpdesk_Ticket::getInstance($params['id']);

            $ticketRow = Catchers_Helpdesk_Addon5_Report::getTicketPublicRow($ticket->getPost(), null);

            $ticketRow['event_type'] = 'change_category';
            $ticketRow['ticket_category'] = $params['category'];
            $ticketRow['ticket_assignee'] = $params['assignee'];

            Catchers_Helpdesk_Addon5_Report::addEventRowToTable($ticketRow);
        });

        add_action('stgh_ticket_assignee_saved', function($params){
            $ticket = Stg_Helpdesk_Ticket::getInstance($params['id']);

            $ticketRow = Catchers_Helpdesk_Addon5_Report::getTicketPublicRow($ticket->getPost(), null);


            $ticketRow['event_type'] = 'change_assignee';
            $ticketRow['user_id'] = $params['assignee'];
            $ticketRow['ticket_category'] = $params['category'];
            $ticketRow['ticket_assignee'] = $params['assignee'];

            Catchers_Helpdesk_Addon5_Report::addEventRowToTable($ticketRow);

        });

        add_action('stgh_ticket_status_saved', function($params){
            $ticket = Stg_Helpdesk_Ticket::getInstance($params['id']);

            $ticketRow = Catchers_Helpdesk_Addon5_Report::getTicketPublicRow($ticket->getPost(), null);

            $ticketRow['event_type'] = 'change_status';
            $ticketRow['ticket_status'] = $params['status'];
            $ticketRow['ticket_category'] = $params['category'];
            $ticketRow['ticket_assignee'] = $params['assignee'];

            Catchers_Helpdesk_Addon5_Report::addEventRowToTable($ticketRow);

        });

        add_action('stgh_ticket_contact_saved', function($params){
            $ticket = Stg_Helpdesk_Ticket::getInstance($params['id']);

            $ticketRow = Catchers_Helpdesk_Addon5_Report::getTicketPublicRow($ticket->getPost(), null);


            $ticketRow['event_type'] = 'change_contact';
            $ticketRow['user_id'] = $params['contact'];
            $ticketRow['ticket_category'] = $params['category'];
            $ticketRow['ticket_assignee'] = $params['assignee'];

            Catchers_Helpdesk_Addon5_Report::addEventRowToTable($ticketRow);

        });

        add_action('stgh_ticket_vote_saved', function($params){
            $ticket = Stg_Helpdesk_Ticket::getInstance($params['id']);

            $ticketRow = Catchers_Helpdesk_Addon5_Report::getTicketPublicRow($ticket->getPost(), null);


            $ticketRow['event_type'] = 'reply_vote';
            $ticketRow['vote'] = $params['vote'];
            $ticketRow['ticket_category'] = $params['category'];
            $ticketRow['ticket_assignee'] = $params['assignee'];

            Catchers_Helpdesk_Addon5_Report::addEventRowToTable($ticketRow);

        });


        add_action('stgh_client_saved', function($params){
            Catchers_Helpdesk_Addon5_Report::addUEventRowToTable($params);
        });

        Catchers_Helpdesk_Addon5_Report_Cron::setCronCreateDefaultReportTasks();

        add_action('admin_menu', array($this, 'add_reports_menu'),8);

        //run updater
		add_action( 'init', array( &$this, 'run_updater' ) );
	}

    public function add_reports_menu()
    {
        add_submenu_page('edit.php?post_type=' . STG_HELPDESK_POST_TYPE,
            __('Reports', STG_HELPDESK_TEXT_DOMAIN_NAME),
            __('Reports', STG_HELPDESK_TEXT_DOMAIN_NAME),
            'edit_ticket',
            'reports',
            array($this,'show_report_page')
        );


    }

    function show_report_page(){
        ob_start();

        /** @noinspection PhpIncludeInspection */
        include_once(STG_CATCHERS_HELPDESK_ADDON5_ROOT . 'admin/partials/reports.php');
        $page =  ob_get_clean();
        echo $page;
    }

	function stgh_create_default_report(){
        ignore_user_abort(true);
        set_time_limit(0);
        Stg_Helpdesk_Init::$isCron = true;

        $haveTasks = Catchers_Helpdesk_Addon5_Report::haveTasks();

        $currentTime = time();

        if($haveTasks == 0) {
            Catchers_Helpdesk_Addon5_Report_Cron::unsetCronCreateDefaultReportTasks();
            return;
        }

        Catchers_Helpdesk_Addon5_Report::clearZombyTasks($currentTime);

        while(time()-$currentTime < 180) {
            Catchers_Helpdesk_Addon5_Report_Cron::cronCreateDefaultReportTasks();
        }

        Stg_Helpdesk_Init::$isCron = false;
    }

	function get_categories()
    {
        $categories = stgh_ticket_get_categories();
        $return = $tmp = array();
        foreach ($categories as $category)
        {
            $tmp['id'] = $category->term_id;
            $tmp['desc'] = $category->name;
            $return[] = $tmp;
        }
        wp_send_json($return);
    }

    function stgh_get_tasks(){
        wp_send_json(array('cnt' => Catchers_Helpdesk_Addon5_Report::haveTasks()));
    }

    function addIndicatorToReturn($indName,$result,$resultPre, $return){
        $return[$indName][0] = $result[$indName]['acc'];
        $return[$indName][1] = stgh_diff_in_pro($result[$indName]['acc'],$resultPre[$indName]['acc']);

        return $return;
    }

    function addComplexIndicatorToReturn($indName,$result,$resultPre, $return, $format = 'acc', $resultKey = null){
        $acc = $accPre = 0;

        if(empty($resultKey))
            $resultKey = $indName;

        foreach($result[$resultKey] as $managersId => $managerStats )
        {
            $acc += $managerStats['acc'];
        }

        foreach($resultPre[$resultKey] as $managersId => $managerStats )
        {
            $accPre += $managerStats['acc'];
        }


        if($format == 'hrhm') {
            if(isset($result[$resultKey]['countAll'])) {
                $indNameValue0 = $acc / $result[$resultKey]['countAll'];
            }else{
                $indNameValue0 = 0;
            }
            if(isset($resultPre[$resultKey]['countAll'])) {
                $indNameValue1 = $accPre / $resultPre[$resultKey]['countAll'];
            }else{
                $indNameValue1 = 0;
            }

            $return[$indName][0] = stgh_convert_ts_to_hrhm($indNameValue0);
            $return[$indName][1] = stgh_diff_in_pro($indNameValue0, $indNameValue1);
        }

        return $return;
    }

    function getManagersIds($result){
        $managersIds = array_unique(
            array_merge(
                array_keys($result['afr']),
                array_keys($result['closedTickets']),
                array_keys($result['totalReplies']),
                array_keys($result['voted'])));
        return $managersIds;
    }

    function addManagerToReturn($indName,$managerId, $result, $resultPre, $format = 'acc'){
        $manager = array();

        $managerStats = $result[$indName][$managerId];

        if($format == 'hrhm'){
            if(isset($managerStats['count']) && $managerStats['count']!=0)
            {
                $avg = $managerStats['acc']/$managerStats['count'];
            }else{
                $avg = 0;
            }

            $manager[0] = stgh_convert_ts_to_hrhm($avg);
            if(isset($resultPre[$indName][$managerId])){
                $manager[1] = stgh_diff_in_pro($avg,$resultPre[$indName][$managerId]['acc']/$resultPre[$indName][$managerId]['count']);
            }else{
                $manager[1] = stgh_diff_in_pro($avg,0);
            }
        }

        if($format == 'count'){
            $manager[0] = (isset($managerStats['count']))? $managerStats['count']:0;

            if(isset($resultPre[$indName][$managerId])){
                $manager[1] = stgh_diff_in_pro($managerStats['count'],$resultPre[$indName][$managerId]['count']);
            }else{
                $manager[1] = stgh_diff_in_pro($managerStats['acc'],0);
            }
        }

        if($format == 'voted'){
            $manager['thegood'] = (isset($managerStats['thegood']))?$managerStats['thegood']:0;
            $manager['thebad'] = (isset($managerStats['thebad']))?$managerStats['thebad']:0;
            $manager['theneutral'] = (isset($managerStats['theneutral']))?$managerStats['theneutral']:0;
        }

        return $manager;
    }

    function addChartsIndicatorToReturn($indName, $result, $return, $type = 'pie'){
        $return['charts'][$indName]['labels'] = $return['charts'][$indName]['data'] = array();

        if($type == 'pie') {
            foreach ($result[$indName] as $indicator) {
                $return['charts'][$indName]['labels'][] = $indicator['name'];
                $return['charts'][$indName]['data'][] = $indicator['count'];
            }
        }

        if($type == 'heat') {
            $return['charts'][$indName]['labels'] = $result[$indName]['heatHours']['labels'];
            unset($result[$indName]['heatHours']);
            $return['charts'][$indName]['data'] = $result[$indName];
        }

        return $return;
    }

    function stgh_get_reports_data(){
	    global $wpdb;
	    $return = array();

        $dateStart = strtotime(filter_input(INPUT_POST, 'stgh_dateStart', FILTER_SANITIZE_STRING));
        $dateEnd = strtotime(filter_input(INPUT_POST, 'stgh_dateEnd', FILTER_SANITIZE_STRING)."+ 1 day");
        $category = (int) filter_input(INPUT_POST, 'stgh_category', FILTER_SANITIZE_STRING);

        $stghClearCache = filter_input(INPUT_POST, 'stgh_clear_cache', FILTER_SANITIZE_STRING);

        $dateStartPre = $dateStart-($dateEnd-$dateStart);
        $mysqli = new mysqli(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);
        $mysqli->set_charset($wpdb->charset);

        $allFilterInd = stgh_reports_filter_reducer($mysqli,$dateStart,$dateEnd,$category,$stghClearCache);
        $allFilterIndPre = stgh_reports_filter_reducer($mysqli,$dateStartPre,$dateStart,$category,$stghClearCache);

        $newContacts = stgh_reports_ufilter_reducer($mysqli,$dateStart,$dateEnd);
        $newContactsPre = stgh_reports_ufilter_reducer($mysqli,$dateStartPre,$dateStart);

        $return['newContacts'][0] = $newContacts;
        $return['newContacts'][1] = stgh_diff_in_pro($return['newContacts'][0],$newContactsPre);


        $return = self::addIndicatorToReturn('newTickets',$allFilterInd,$allFilterIndPre, $return);
        $return = self::addIndicatorToReturn('closedTickets',$allFilterInd,$allFilterIndPre, $return);
        $return = self::addIndicatorToReturn('totalReplies',$allFilterInd,$allFilterIndPre, $return);

        $return = self::addComplexIndicatorToReturn('averageFirst',$allFilterInd,$allFilterIndPre, $return,'hrhm','afr');
        $return = self::addComplexIndicatorToReturn('averageClosing',$allFilterInd,$allFilterIndPre, $return,'hrhm', 'act');


        $managersIds = self::getManagersIds($allFilterInd);

        $return['managers'] = $manager = array();

        foreach($managersIds as $managerId)
        {

            if(!is_numeric($managerId))
                continue;

            if(!$managerId>0)
                continue;

            $return['managers'][$managerId]['averageFirst'] = self::addManagerToReturn('afr',$managerId, $allFilterInd, $allFilterIndPre, 'hrhm');
            $return['managers'][$managerId]['closedTickets'] = self::addManagerToReturn('act',$managerId, $allFilterInd, $allFilterIndPre, 'count');
            $return['managers'][$managerId]['totalReplies'] = self::addManagerToReturn('totalReplies',$managerId, $allFilterInd, $allFilterIndPre,'count');
            $return['managers'][$managerId]['voted'] = self::addManagerToReturn('voted',$managerId, $allFilterInd, $allFilterIndPre, 'voted');

            $user = get_user_by('ID',$managerId);

            $return['managers'][$managerId]['avatar'] = get_avatar_url($managerId);
            $return['managers'][$managerId]['userNameLink'] = get_edit_user_link($managerId);
            if($user) {
                $return['managers'][$managerId]['userName'] = $user->display_name;
            }else{
                $noname =  sprintf(
                        '%s (# %d)',
                        _x('deleted', 'label of undefined indicator', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
                        $managerId,
                        false);

                $return['managers'][$managerId]['userName'] = $noname;
            }
        }



        $return = self::addChartsIndicatorToReturn('categories',$allFilterInd, $return);
        $return = self::addChartsIndicatorToReturn('channels',$allFilterInd, $return);
        $return = self::addChartsIndicatorToReturn('heat',$allFilterInd, $return, 'heat');

        $openTickets = $allFilterInd['opentickets'];

        $return['charts']['opentickets']['stgh_new'] = array('data' => array_values($openTickets['stgh_new']),'labels'=>array_keys($openTickets['labels']));
        $return['charts']['opentickets']['stgh_answered'] = array('data' => array_values($openTickets['stgh_answered']),'labels'=>array_keys($openTickets['labels']));
        $return['charts']['opentickets']['stgh_notanswered'] = array('data' => array_values($openTickets['stgh_notanswered']),'labels'=>array_keys($openTickets['labels']));

        wp_send_json($return);
    }


    function stgh_core_helpcatcher($def)
    {
        ob_start();

        /** @noinspection PhpIncludeInspection */
        include_once(STG_CATCHERS_HELPDESK_ADDON5_ROOT . 'admin/partials/reports.php');
        $page =  ob_get_clean();


        $settings = array(
            'reports' => array(
                'name' => __('Reports', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME),
                'stgh-no-save-button' => true,
                'options' => array(
                    array(
                        'id' => 'stgh_reports',
                        'type' => 'custom',
                        'custom' => $page,
                        'default' => '',
                    ),
                )
            ),
        );

        return array_merge($def, $settings);

    }

	function registerMenu(){
			add_submenu_page('index.php',
				__('Catchers Helpdesk  Add-on starting page.', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME), __('Welcome', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME), 'activate_plugins',
				'stgh-addon5-start', array($this, 'showWelcomePage'));

			remove_submenu_page('index.php', 'stgh-addon5-start');
	}

	function redirectToWelcomePage(){
		if(get_option('stgh_redirect_to_welcome_page_addon'))
		{
			wp_redirect( add_query_arg(array('page' => 'stgh-addon5-start'),admin_url('index.php')));
			delete_option('stgh_redirect_to_welcome_page_addon');
		}
	}

	public function showWelcomePage()
	{
		/** @noinspection PhpIncludeInspection */
		include_once(STG_CATCHERS_HELPDESK_ADDON5_ROOT . 'admin/partials/welcome.php');
	}

	public function pluginRowMeta( $links, $file ) {
		list($part1,$part2) = explode('/',$file);

		if ( STG_CATCHERS_HELPDESK_ADDON5_NAME == $part1 &&  STG_CATCHERS_HELPDESK_ADDON5_NAME.".php" == $part2) {

			$addonDeps = Catchers_Helpdesk_Addon5::checkPluginDeps();
			$freeHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME.'/'.STG_CATCHERS_HELPDESK_NAME.'.php');
			$proHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME_PRO . '/' . STG_CATCHERS_HELPDESK_NAME_PRO . '.php');
			$welcomePage = add_query_arg(array('page' => 'stgh-addon5-start'),admin_url('index.php'));
			$row_meta = array();

			if(!$freeHDActivated && !$proHDActivated){
				$row_meta = array(
					'welcome'    => '<a class="error-message" href="' . esc_url($welcomePage) . '" >' . esc_html__( 'Attention, please! Catchers Helpdesk plugin is not installed or not activated!', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME ) . '</a>'
				);

			}else{
				if (!$addonDeps) {
					$row_meta = array(
						'welcome'    => '<a class="error-message" href="' . esc_url($welcomePage) . '">' . esc_html__( 'Attention, please! You have the old version of Catchers Helpdesk!', STG_CATCHERS_HELPDESK_ADDON5_TEXT_DOMAIN_NAME ) . '</a>'
					);
				}
			}

			return array_merge( $links, $row_meta );
		}

		return (array) $links;
	}


	function run_updater(){
		$pluginPath = STG_CATCHERS_HELPDESK_ADDON5_ROOT."".STG_CATCHERS_HELPDESK_ADDON5_NAME.".php";
		$pluginData = stgh_wc_updater_get_plugin_data($pluginPath);
		$pluginCurrentVersion = $pluginData['Version'];

		$adapterName = 'Mycatchers';
		$data = array(
			'Mycatchers' => array(
				'license' => 'none',
			)
		);

        $pluginLicense = false;
		if(function_exists('stgh_get_option')) {
            $pluginLicense = stgh_get_option('plugin_license');
            $envatoToken = stgh_get_option('envato_token');
        }

		if(strlen($pluginLicense) == 34){
			$adapterName = 'Mycatchers';
			$data = array(
				'Mycatchers' => array(
					'license' => $pluginLicense
				)
			);
		}else{
			if(!empty($envatoToken))
			{
				$adapterName = 'Envato';
				$data = array(
					'Envato' => array(
						'license' => 'envato',
						'token' => $envatoToken,
						//@todo
						'itemId' => ''
					)
				);
			}
		}
		if($adapterName != false){
			if(!class_exists('\StgHelpdeskAddon5\Updater\AutoUpdater'))
			{
				/** @noinspection PhpIncludeInspection */
				require_once plugin_dir_path( dirname( __FILE__ ) ) . 'updater/auto_updater.php';
			}
			new \StgHelpdeskAddon5\Updater\AutoUpdater($adapterName, $pluginCurrentVersion, $data, $this->get_plugin_name());
		}

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Catchers_Helpdesk_Addon5_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Catchers_Helpdesk_Addon5_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

	public static function checkPluginDeps()
	{
		include_once(ABSPATH.'wp-admin/includes/plugin.php');

		$freeHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME . '/' . STG_CATCHERS_HELPDESK_NAME . '.php');
        $proHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME_PRO . '/' . STG_CATCHERS_HELPDESK_NAME_PRO . '.php');

		if (!$freeHDActivated && !$proHDActivated) {
			return false;
		} else {
		    if(defined('STG_HELPDESK_ROOT') && defined('STG_HELPDESK_NAME')) {
                $pluginFile = STG_HELPDESK_ROOT . STG_HELPDESK_NAME . ".php";
                $readmeFile = STG_HELPDESK_ROOT . "README.txt";
                $pluginData = stgh_wc_updater_get_plugin_data($pluginFile, $readmeFile);

                if (version_compare($pluginData["Version"], STG_CATCHERS_HELPDESK_ADDON5_VERSION, '<')) {
                    return false;
                }
            }

		}

		return true;
	}

}
