<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://mycatchers.com
 * @since      1.0.0
 *
 * @package    Catchers_Helpdesk_Addon2
 * @subpackage Catchers_Helpdesk_Addon2/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Catchers_Helpdesk_Addon2
 * @subpackage Catchers_Helpdesk_Addon2/includes
 * @author     mycatchers <support@mycatchers.com>
 */
class Catchers_Helpdesk_Addon2 {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Catchers_Helpdesk_Addon2_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct($pluginName) {

		$this->plugin_name = $pluginName;
		$this->version = STG_CATCHERS_HELPDESK_ADDON2_VERSION;


		$this->load_dependencies();
		$this->set_locale();

		$depStatus = self::checkPluginDeps();

		if(!$depStatus) {
            define('STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME', 'catchers-helpdesk');
			add_filter('plugin_row_meta', array($this, 'pluginRowMeta'), 10, 2);
			add_action('admin_init', array($this, 'redirectToWelcomePage'), 12, 0);
			add_action('admin_menu', array($this, 'registerMenu'));
		}
        else{
            if(defined('STG_HELPDESK_POST_TYPE') && $depStatus){
                define('STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME', STG_HELPDESK_TEXT_DOMAIN_NAME);
                $this->define_admin_hooks();
                $this->define_public_hooks();
            }
        }



	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Catchers_Helpdesk_Addon2_Loader. Orchestrates the hooks of the plugin.
	 * - Catchers_Helpdesk_Addon2_i18n. Defines internationalization functionality.
	 * - Catchers_Helpdesk_Addon2_Admin. Defines all hooks for the admin area.
	 * - Catchers_Helpdesk_Addon2_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-catchers-helpdesk-addon2-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-catchers-helpdesk-addon2-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-catchers-helpdesk-addon2-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-catchers-helpdesk-addon2-public.php';

		$this->loader = new Catchers_Helpdesk_Addon2_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Catchers_Helpdesk_Addon2_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Catchers_Helpdesk_Addon2_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Catchers_Helpdesk_Addon2_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		$helpdeskInit = \StgHelpdesk\Core\Stg_Helpdesk_Init::getInstance();

		//add saved replies
		add_action('init', array($helpdeskInit, 'create_taxonomy_savedreplies'));

		//add tinymce in saved_replies
		add_filter('savedreply_edit_form_fields', array($helpdeskInit, 'edit_cat_description'));
		add_action('admin_head', array($helpdeskInit, 'taxonomy_tinycme_hide_description'));
		remove_filter('pre_term_description', 'wp_filter_kses');
		remove_filter('term_description', 'wp_kses_data');
		add_filter('savedreply_row_actions', array($helpdeskInit, 'remove_row_actions'), 10, 2 );

		add_action('catcher-helpdesk-ticket-ntabs', array($this,'add_private_note_tab'));
		add_action('catcher-helpdesk-ticket-ccbcc-toggle', array($this,'add_ccbcc_link'));
		add_action('catcher-helpdesk-ticket-ccbcc', array($this,'add_ccbcc'));
		add_action('catcher-helpdesk-ticket-history',array($this,'add_history'),10,1);

		add_filter('stgh_plugin_settings', array($this,'stgh_core_settings'), 96, 1);

		add_filter('catcher-helpdesk-comments-filter', array($this,'stgh_comments_filter_private'));
		add_filter('catcher-helpdesk-comments-filter-menu', array($this,'stgh_comments_filter_menu'));
		add_filter('stgh_plugin_settings', array($this,'stgh_updates_integrations'), 100, 1);
        add_filter('stgh_plugin_settings', array($this,'stgh_ticket_vote_for_close_core'), 102, 1);
        add_filter('stgh_plugin_settings', array($this,'stgh_updates_notification'), 100, 1);


        //add ticket voting form to macros
        add_filter('stgh_saved_replies_macros', array($this,'stgh_saved_replies_macros'));

        add_filter('stgh_table_column', array($this,'stgh_table_column'));

		//run updater
		add_action( 'init', array( &$this, 'run_updater' ) );
	}

	function stgh_saved_replies_macros($macros){
	    if(isset($macros[2]) && $macros[2]['id'] == 'ticket')
	        $macros[2]['children'][] = array('text' => __('Reply Voting Form', STG_HELPDESK_TEXT_DOMAIN_NAME), 'id' => '{%ticket.voting_form%}');
	    return $macros;
    }

    function stgh_table_column($columns){
        $columns[STG_HELPDESK_POST_TYPE . '-vote'] = __('Vote', STG_HELPDESK_TEXT_DOMAIN_NAME);
        return $columns;
    }

    function stgh_ticket_vote_for_close_core($def){

        $settings = array(
            array(
                'name' => __('Check customer satisfaction', STG_HELPDESK_TEXT_DOMAIN_NAME),
                'desc' => __('Send a voting message to the customer when the ticket is closed', STG_HELPDESK_TEXT_DOMAIN_NAME),
                'id' => 'stg_ticket_vote_for_close',
                'type' => 'checkbox',
                'default' => stgh_get_option('stg_ticket_vote_for_close', false),
            ));

        $def['core']['options'] = array_merge(
            array_slice($def['core']['options'], 0, 2),
            $settings,
            array_slice($def['core']['options'], 2)
        );

        return $def;

    }

	function stgh_updates_integrations($def)
	{

        foreach($def['email']['options'] as $key => $option)
        {
            if($option['id'] == 'ticket_subject')
            {
                $def['email']['options'][$key]['hidden'] = false;
            }

        }

		$settings = array(
			'integrations' => array(
				'name' => __('Integrations', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME),
				'options' => array(
					array(
						'type' => 'heading',
						'name' => __('Addon\'s update', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME)
					),
					array(
						'name' => __('Envato Personal Token', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME),
						'id' => 'envato_token',
						'type' => 'text',
					),
					array(
						'name' => __('Plugin license', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME),
						'id' => 'plugin_license',
						'type' => 'text',
					),
				)
			),
		);

		$res = array_merge($def, $settings);

		if(isset($def['integrations']['options']))
		{
			$res['integrations']['options'] = array_unique(array_merge($def['integrations']['options'], $settings['integrations']['options']), SORT_REGULAR);
		}

		return $res;

	}

    function stgh_updates_notification($def)
    {

        $settings = array(
            'email' => array(
                'name' => __('Notifications', STG_HELPDESK_TEXT_DOMAIN_NAME),
                'options' => array(
                    array(
                        'name' => __('Global Cc for all notifications', STG_HELPDESK_TEXT_DOMAIN_NAME),
                        'type' => 'text',
                        'id' => 'stg_global_cc',
                        'desc' => __('e-mail address for set Cc header for all notification', STG_HELPDESK_TEXT_DOMAIN_NAME)

                    ),
                    array(
                        'name' => __('Global Bcc for all notifications', STG_HELPDESK_TEXT_DOMAIN_NAME),
                        'type' => 'text',
                        'id' => 'stg_global_bcc',
                        'desc' => __('e-mail address for set Bcc header for all notification', STG_HELPDESK_TEXT_DOMAIN_NAME)

                    ),
                )
            ),
        );

        $res = array_merge($def, $settings);

        if(isset($def['email']['options']))
        {
            $res['email']['options'] = array_unique(array_merge($def['email']['options'], $settings['email']['options']), SORT_REGULAR);
        }

        return $res;

    }

	function stgh_comments_filter_menu($items){
		$addItems = array('private' => __('Private notes', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME), 'history' => __('History', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME), 'vote' => __('Vote', STG_HELPDESK_TEXT_DOMAIN_NAME));
		return array_merge($items,$addItems);
	}

	function stgh_comments_filter_private($params){

		foreach($params['meta_query'] as $key => $meta)
		{
			if(isset($meta["key"]) && isset($meta["value"]) && isset($meta["compare"])) {
				if ($meta["key"] == "_stgh_type_color" && ($meta["value"] == "private" || $meta["value"] == "history") && $meta["compare"] == "!=") {
					unset($params['meta_query'][$key]);
				}
			}
		}

		return $params;
	}


	function add_history($ticket_event_meta){
		?>
		<div class="stgh-ticket-event">
			<ul>
				<?php if (!empty($ticket_event_meta['assignee'])) : ?>
					<li><label
							class="stgh-text-bold"><?php _e('Assignee', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME); ?></label> <?= $ticket_event_meta['assignee']; ?>
					</li>
				<?php endif; ?>
				<?php if (!empty($ticket_event_meta['subject'])) : ?>
					<li><label
							class="stgh-text-bold"><?php _e('Subject', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME); ?></label> <?= $ticket_event_meta['subject']; ?>
					</li>
				<?php endif; ?>
				<?php if (!empty($ticket_event_meta['status'])) : ?>
					<li><label
							class="stgh-text-bold"><?php _e('Status', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME); ?></label> <?= $ticket_event_meta['status']; ?>
					</li>
				<?php endif; ?>
				<?php if (!empty($ticket_event_meta['contact'])) : ?>
					<li><label
							class="stgh-text-bold"><?php _e('Contact', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME); ?></label> <?= $ticket_event_meta['contact']; ?>
					</li>
				<?php endif; ?>
				<?php if (!empty($ticket_event_meta['category'])) : ?>
					<li><label
							class="stgh-text-bold"><?php _e('Category', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME); ?></label> <?= $ticket_event_meta['category']; ?>
					</li>
				<?php endif; ?>
				<?php if (!empty($ticket_event_meta['tags'])) : ?>
					<li><label
							class="stgh-text-bold"><?php _e('Tags', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME); ?></label> <?= $ticket_event_meta['tags']; ?>
					</li>
				<?php endif; ?>
                <?php if (!empty($ticket_event_meta['vote'])) : ?>
                    <li><label
                                class="stgh-text-bold"><?php _e('Vote', STG_HELPDESK_TEXT_DOMAIN_NAME); ?></label> <?= $ticket_event_meta['vote']; ?>
                    </li>
                <?php endif; ?>
			</ul>
		</div>
		<?php
	}

	function add_ccbcc_link(){
		?>
		<a id="toggleCcBcc"><?php _e('Cc/Bcc', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME); ?></a>
		<?php
	}

	function add_ccbcc(){
		?>
		<p class="stgh-cc-bcc">
			<label for="stgh-cc"><strong><?php _e('Cc:', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME); ?></strong></label>
			<input type="text" value=""  name="stgh-cc" id="stgh-cc">
		</p>

		<p class="stgh-cc-bcc">
			<label for="stgh-bcc"><strong><?php _e('Bcc:', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME); ?></strong></label>
			<input type="text" value=""  name="stgh-bcc" id="stgh-bcc">
		</p>
		<?php
	}


	function stgh_core_settings($def)
	{

		$settings = array(
			'core' => array(
				'name' => __('General', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME),
				'options' => array(
					array(
						'name' => __('Default assignee', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME),
						'id' => 'assignee_default',
						'type' => 'select',
						'desc' => __('New tickets will be automatically assigned to that person', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME),
						'options' => stgh_list_users_manager(),
						'default' => ''
					),
					array(
						'name' => __('File Upload Configuration', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME),
						'id' => 'stgh_enable_attachment',
						'type' => 'checkbox',
						'default' => stgh_get_option('stgh_enable_attachment', false),
						'desc' => __('Do you want to allow users and helpdesk managers to upload attachments? <br> (Option is also used for file send through the mail server)', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME)
					),
					array(
						'name' => __('Tracking', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME),
						'type' => 'heading',
					),
					array(
						'name' => __('Open tracking', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME),
						'id' => 'open_tracking',
						'type' => 'checkbox',
						'default' => false,
						'desc' => __('Enable open tracking', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME)
					),
					array(
						'name' => __('Auto reply', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME),
						'type' => 'heading',
					),
					array(
						'name' => __('Status', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME),
						'id' => 'enable_open',
						'type' => 'checkbox',
						'default' => stgh_get_option('enable_auto_reply', false),
						'desc' => __('Enable auto reply', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME)
					),

					array(
						'name' => __('Message', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME),
						'hidden' => true,
						'id' => 'content_auto_reply',
						'type' => 'editor',
						'default' => __('<br /><p>Hello, <br /> We just got your help request! And do our best to answer emails as soon as possible, with most inquiries receiving a response within about a day.</p>', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME),
						'editor_settings' => array('quicktags' => true, 'textarea_rows' => 10, 'wpautop' => false, 'teeny' => true),
					),
                array(
                    'name' => __('Email footer', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME),
                    'type' => 'heading',
                ),
                array(
                    'name' => __('Footer content', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME),
                    'id' => 'stgh_email_footer_content',
                    'type' => 'editor',
                    'default' => '',
                    'editor_settings' => array('quicktags' => true, 'textarea_rows' => 10, 'wpautop' => false, 'teeny' => true),

                ),
                array(
                    'name' => __('Plugin pages', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME),
                    'type' => 'heading',
                ),
                array(
                    'name' => __('Create plugin pages', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME),
                    'type' => 'custom',
                    'custom' => '<a id="stgh_chk_conf" class="button" href="' .  stgh_link_to_create_user_pages() . '">' . __('Create',	STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME) . '</a> <img class="stgh_loader_image" hidden id="stgh_loader" src="' . STG_HELPDESK_URL . 'images/ajax-loader.gif" >',
                ),
                array(
                    'name' => __('Add headers to emails body', STG_HELPDESK_TEXT_DOMAIN_NAME),
                    'id' => 'add_header_to_ebody',
                    'type' => 'checkbox',
                    'default' => stgh_get_option('add_header_to_ebody', false)
                ),
				)
			),
		);

		return array_merge($def, $settings);

	}


	function add_private_note_tab(){
		?>
		<li class="stgh-ntab stgh-private-note" data-show="stgh-form-private"><?php _e('Private note', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME); ?>
		<?php
	}


	function registerMenu(){
			add_submenu_page('index.php',
				__('Catchers Helpdesk  Add-on starting page.', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME), __('Welcome', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME), 'activate_plugins',
				'stgh-addon2-start', array($this, 'showWelcomePage'));

			remove_submenu_page('index.php', 'stgh-addon2-start');
	}

	function redirectToWelcomePage(){
		if(get_option('stgh_redirect_to_welcome_page_addon'))
		{
			wp_redirect( add_query_arg(array('page' => 'stgh-addon2-start'),admin_url('index.php')));
			delete_option('stgh_redirect_to_welcome_page_addon');
		}
	}

	public function showWelcomePage()
	{
		/** @noinspection PhpIncludeInspection */
		include_once(STG_CATCHERS_HELPDESK_ADDON2_ROOT . 'admin/partials/welcome.php');
	}

	public function pluginRowMeta( $links, $file ) {
		list($part1,$part2) = explode('/',$file);

		if ( STG_CATCHERS_HELPDESK_ADDON2_NAME == $part1 &&  STG_CATCHERS_HELPDESK_ADDON2_NAME.".php" == $part2) {

			$addonDeps = Catchers_Helpdesk_Addon2::checkPluginDeps();
			$freeHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME.'/'.STG_CATCHERS_HELPDESK_NAME.'.php');

			$welcomePage = add_query_arg(array('page' => 'stgh-addon2-start'),admin_url('index.php'));
			$row_meta = array();

			if(!$freeHDActivated){
				$row_meta = array(
					'welcome'    => '<a class="error-message" href="' . esc_url($welcomePage) . '" >' . esc_html__( 'Attention, please! Catchers Helpdesk plugin is not installed or not activated!', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME ) . '</a>'
				);

			}else{
				if (!$addonDeps) {
					$row_meta = array(
						'welcome'    => '<a class="error-message" href="' . esc_url($welcomePage) . '">' . esc_html__( 'Attention, please! You have the old version of Catchers Helpdesk!', STG_CATCHERS_HELPDESK_ADDON2_TEXT_DOMAIN_NAME ) . '</a>'
					);
				}
			}

			return array_merge( $links, $row_meta );
		}

		return (array) $links;
	}


	function run_updater(){
		$pluginPath = STG_CATCHERS_HELPDESK_ADDON2_ROOT."".STG_CATCHERS_HELPDESK_ADDON2_NAME.".php";
		$pluginData = stgh_wc_updater_get_plugin_data($pluginPath);
		$pluginCurrentVersion = $pluginData['Version'];

		$adapterName = 'Mycatchers';
		$data = array(
			'Mycatchers' => array(
				'license' => 'none',
			)
		);


		$pluginLicense = stgh_get_option('plugin_license');
		$envatoToken = stgh_get_option('envato_token');


		if(strlen($pluginLicense) == 34){
			$adapterName = 'Mycatchers';
			$data = array(
				'Mycatchers' => array(
					'license' => $pluginLicense
				)
			);
		}else{
			if(!empty($envatoToken))
			{
				$adapterName = 'Envato';
				$data = array(
					'Envato' => array(
						'license' => 'envato',
						'token' => $envatoToken,
						//@todo
						'itemId' => ''
					)
				);
			}
		}
		if($adapterName != false){
			if(!class_exists('\StgHelpdeskAddon2\Updater\AutoUpdater'))
			{
				/** @noinspection PhpIncludeInspection */
				require_once plugin_dir_path( dirname( __FILE__ ) ) . 'updater/auto_updater.php';
			}
			new \StgHelpdeskAddon2\Updater\AutoUpdater($adapterName, $pluginCurrentVersion, $data, $this->get_plugin_name());
		}

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Catchers_Helpdesk_Addon2_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Catchers_Helpdesk_Addon2_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

	public static function checkPluginDeps()
	{
		include_once(ABSPATH.'wp-admin/includes/plugin.php');

		$freeHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME . '/' . STG_CATCHERS_HELPDESK_NAME . '.php');

		if (!$freeHDActivated) {
			return false;
		} else {
			$pluginFile = STG_HELPDESK_ROOT.STG_HELPDESK_NAME.".php";
			$readmeFile = STG_HELPDESK_ROOT."README.txt";
			$pluginData = stgh_wc_updater_get_plugin_data($pluginFile, $readmeFile);

			if (version_compare($pluginData["Version"], STG_CATCHERS_HELPDESK_ADDON2_VERSION, '<')) {
				return false;
			}

		}

		return true;
	}

}
