<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://mycatchers.com
 * @since      1.0.0
 *
 * @package    Catchers_WC_Support
 * @subpackage Catchers_WC_Support/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Catchers_WC_Support
 * @subpackage Catchers_WC_Support/includes
 * @author     Your Name <email@example.com>
 */
class Catchers_WC_Support {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Catchers_WC_Support_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct($pluginName) {

		$this->plugin_name = $pluginName;
		$this->version = '1.0.0';

		$this->load_dependencies();
		$this->set_locale();

		$depStatus = self::checkPluginDeps();

		if(defined('STG_HELPDESK_POST_TYPE') && is_plugin_active('woocommerce/woocommerce.php') && $depStatus){
            define('STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME', STG_HELPDESK_TEXT_DOMAIN_NAME);
		    $this->define_admin_hooks();
			$this->define_public_hooks();
		}else{
            define('STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME', 'catchers-helpdesk');
			add_action('admin_init', array($this, 'redirectToWelcomePage'), 12, 0);
			add_action('admin_menu', array($this, 'registerMenu'));
			add_filter( 'plugin_row_meta', array($this, 'pluginRowMeta' ), 10, 2 );
		}
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Catchers_WC_Support_Loader. Orchestrates the hooks of the plugin.
	 * - Catchers_WC_Support_i18n. Defines internationalization functionality.
	 * - Catchers_WC_Support_Admin. Defines all hooks for the admin area.
	 * - Catchers_WC_Support_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-catchers-wc-support-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-catchers-wc-support-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-catchers-wc-support-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		/** @noinspection PhpIncludeInspection */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-catchers-wc-support-public.php';

		$this->loader = new Catchers_WC_Support_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Catchers_WC_Support_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Catchers_WC_Support_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Catchers_WC_Support_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		add_filter('stgh_plugin_settings', array($this,'stgh_updates_integrations'), 99, 1);

		$this->loader->add_action( 'add_meta_boxes', null, 'stg_wc_register_metaboxes', 11 );
		$this->loader->add_action( 'stg_free_form_template_include', null, 'stg_wc_add_product_placeholder', 11 );
		$this->loader->add_action( 'stg_get_customfield_result', null, 'stg_wc_add_products_in_selector', 11 , 1);
		$this->loader->add_action( 'stg_get_form_text_before', null, 'stg_wc_remove_products_from_array', 11 , 1);
		$this->loader->add_action( 'stg_get_form_text_replaces', null, 'stg_wc_form_text_add_replace', 11 , 1);
		$this->loader->add_filter( 'woocommerce_general_settings', null, 'stg_wc_add_general_settings', 11 , 1);
		$this->loader->add_action( 'woocommerce_settings_save_general', null, 'stg_wc_add_general_settings_save', 11 , 2);

		$this->loader->add_action( 'stgh_agent_crm_right', null, 'stg_wc_add_order_list_box' );


		//run updater
		add_action( 'init', array( &$this, 'run_updater' ) );
	}

	function registerMenu(){

		add_submenu_page('index.php',
			__('Catchers Helpdesk  Add-on starting page. ', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME), __('Welcome', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME), 'activate_plugins',
			'stgh-wc-addon-start', array($this, 'showWelcomePage'));

		remove_submenu_page('index.php', 'stgh-wc-addon-start');
	}

	function redirectToWelcomePage(){
		if(get_option('stgh_redirect_to_welcome_page_wc_addon'))
		{
			wp_redirect( add_query_arg(array('page' => 'stgh-wc-addon-start'),admin_url('index.php')));
			delete_option('stgh_redirect_to_welcome_page_wc_addon');
		}
	}

	public function showWelcomePage()
	{
		/** @noinspection PhpIncludeInspection */
		include_once(STG_HELPDESK_WC_SUPPORT_ROOT . 'admin/partials/welcome.php');
	}

	public static function checkPluginDeps()
	{
		include_once(ABSPATH.'wp-admin/includes/plugin.php');

		$freeHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME . '/' . STG_CATCHERS_HELPDESK_NAME . '.php');
		$proHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME_PRO . '/' . STG_CATCHERS_HELPDESK_NAME_PRO . '.php');

		if (!$freeHDActivated && !$proHDActivated) {
			return false;
		} else {
			$pluginFile = STG_HELPDESK_ROOT.STG_HELPDESK_NAME.".php";
			$readmeFile = STG_HELPDESK_ROOT."README.txt";
			$pluginData = stgh_wc_updater_get_plugin_data($pluginFile, $readmeFile);

			if (version_compare($pluginData["Version"], STG_HELPDESK_WC_SUPPORT_VERSION, '<')) {
				return false;
			}

		}

		return true;
	}


	public function pluginRowMeta( $links, $file )
	{
		list($part1,$part2) = explode('/',$file);
		if ( STG_HELPDESK_WC_SUPPORT_NAME == $part1 &&  STG_HELPDESK_WC_SUPPORT_NAME.".php" == $part2) {

			$freeHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME.'/'.STG_CATCHERS_HELPDESK_NAME.'.php');
			$proHDActivated = is_plugin_active(STG_CATCHERS_HELPDESK_NAME_PRO . '/' . STG_CATCHERS_HELPDESK_NAME_PRO . '.php');
			$addonDeps = Catchers_WC_Support::checkPluginDeps();

			$welcomePage = add_query_arg(array('page' => 'stgh-wc-addon-start'),admin_url('index.php'));
			$row_meta = array();

			if(!$freeHDActivated && !$proHDActivated){
				$row_meta = array(
					'welcome'    => '<a class="error-message" href="' . esc_url($welcomePage) . '" >' . esc_html__( 'Attention, please! Catchers Helpdesk plugin is not installed or not activated!', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME ) . '</a>'
				);

			}else{
				if(!$addonDeps)
				{
					$row_meta = array(
						'welcome'    => '<a class="error-message" href="' . esc_url($welcomePage) . '" >' . esc_html__( 'Attention, please! You have the old version of Catchers Helpdesk!', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME ) . '</a>'
					);
				}
			}

			$row_meta2 = array();
			if(!is_plugin_active('woocommerce/woocommerce.php')){
				$row_meta2 = array(
					'welcome2'    => '<a class="error-message" href="' . esc_url($welcomePage) . '" >' . esc_html__( 'Woocommerce not installed or not activated!', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME ) . '</a>'
				);

			}


			return array_merge( $links, $row_meta,$row_meta2 );
		}

		return (array) $links;
	}



	function run_updater(){
		$pluginPath = STG_HELPDESK_WC_SUPPORT_ROOT."".STG_HELPDESK_WC_SUPPORT_NAME.".php";
		$pluginData = stgh_wc_updater_get_plugin_data($pluginPath);
		$pluginCurrentVersion = $pluginData['Version'];

		$adapterName = 'Mycatchers';
		$data = array(
			'Mycatchers' => array(
				'license' => 'none',
			)
		);


		$pluginLicense = stgh_get_option('plugin_license');
		$envatoToken = stgh_get_option('envato_token');


		if(strlen($pluginLicense) == 34){
			$adapterName = 'Mycatchers';
			$data = array(
				'Mycatchers' => array(
					'license' => $pluginLicense
				)
			);
		}else{
			if(!empty($envatoToken))
			{
				$adapterName = 'Envato';
				$data = array(
					'Envato' => array(
						'license' => 'envato',
						'token' => $envatoToken,
						'itemId' => '19977596'
					)
				);
			}
		}
		if($adapterName != false){
			if(!class_exists('\StgHelpdeskNew\Updater\AutoUpdater'))
			{
				/** @noinspection PhpIncludeInspection */
				require_once plugin_dir_path( dirname( __FILE__ ) ) . 'updater/auto_updater.php';
			}
			new \StgHelpdeskNew\Updater\AutoUpdater($adapterName, $pluginCurrentVersion, $data, $this->get_plugin_name());
		}

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Catchers_WC_Support_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

		$this->loader->add_filter( 'woocommerce_my_account_my_orders_actions', null, 'stg_wc_custom_orders_fields', 10, 2 );
		$this->loader->add_action('wp_ajax_stgh_get_user_form', null, 'stg_wc_get_user_form');
		$this->loader->add_action('wp_ajax_stgh_get_product_search', null, 'stgh_ajax_get_product_search');
        $this->loader->add_action('wp_ajax_nopriv_stgh_get_product_search', null, 'stgh_ajax_get_product_search');
		$this->loader->add_action('save_post_' . STG_HELPDESK_POST_TYPE, null, 'stg_wc_save_ticket_meta', 11, 2);

		$this->loader->add_action( 'init', null, 'stg_wc_tickets_endpoint' );
		$this->loader->add_filter( 'query_vars', null, 'stg_wc_tickets_query_vars', 0 );
		$this->loader->add_filter( 'woocommerce_account_menu_items', null, 'stg_wc_tickets_link_my_account' );
		$this->loader->add_action( 'woocommerce_account_my_tickets_endpoint', null, 'stg_wc_tickets_content' );
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Catchers_WC_Support_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

	function stgh_updates_integrations($def)
	{

		$settings = array(
			'integrations' => array(
				'name' => __('Integrations', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME),
				'options' => array(
					array(
						'type' => 'heading',
						'name' => __('Addon\'s update', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME)
					),
					array(
						'name' => __('Envato Personal Token', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME),
						'id' => 'envato_token',
						'type' => 'text',
					),
					array(
						'name' => __('Plugin license', STG_HELPDESK_WC_SUPPORT_TEXT_DOMAIN_NAME),
						'id' => 'plugin_license',
						'type' => 'text',
					),
				)
			),
		);

		$res = array_merge($def, $settings);

		if(isset($def['integrations']['options']))
		{
			$res['integrations']['options'] = array_unique(array_merge($def['integrations']['options'], $settings['integrations']['options']), SORT_REGULAR);
		}

		return $res;

	}

}
